import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:boring_widget/boring_widget.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';
import 'package:analyzer/dart/element/visitor.dart';
import 'package:analyzer/dart/element/type.dart';

class BoringWidgetGenerator extends GeneratorForAnnotation<BoringWidget> {
  const BoringWidgetGenerator();

  @override
  FutureOr<String> generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) {
    return _generateWidgetSource(element);
  }

  String _generateWidgetSource(Element element) {
    final visitor = ModelVisitor();
    element.visitChildren(visitor);

    var classBuffer = StringBuffer();
    var widgetName = '${visitor.className}Widget';

    classBuffer.writeln('class $widgetName extends StateLessWidget {');
    classBuffer.writeln('$widgetName(');
    for (var parameterName in visitor.fields.keys) {
      classBuffer.writeln('this.$parameterName,');
    }
    classBuffer.writeln(');');
    classBuffer.writeln('}');
    return '/*' + classBuffer.toString() + '*/';
  }
}

class ModelVisitor extends SimpleElementVisitor {
  DartType className;
  Map<String, DartType> fields;

  @override
  visitConstructorElement(ConstructorElement element) {
    assert(className == null);
    className = element.type.returnType;
  }

  @override
  visitFieldElement(FieldElement element) {
    fields[element.name] = element.type;

    return super.visitFieldElement(element);
  }
}
